import unittest


class TestUtils(unittest.TestCase):
    """Test our utility functions.

    These should be plain unit tests without layers, so you can execute them
    with one of these lines, depending on your setup:

    * bin/test -u
    * tox -e py311-Plone60 -- -u
    * .tox/py311-Plone60/bin/test -u
    """

    def test_parse_data_tile(self):
        from collective.tileindex.utils import parse_data_tile

        # These are common ones:
        self.assertEqual(
            parse_data_tile("./@@plone.app.standardtiles.html/some-tile-uid"),
            "plone.app.standardtiles.html",
        )
        self.assertEqual(
            parse_data_tile(
                "./@@plone.app.standardtiles.field?field=IDublinCore-title"
            ),
            "plone.app.standardtiles.field",
        )

        # I can imagine some variants that we want to accept:
        self.assertEqual(
            parse_data_tile("./@@example.tile1/some-tile-uid?field=x"),
            "example.tile1",
        )
        self.assertEqual(
            parse_data_tile("./@@example.tile2/"),
            "example.tile2",
        )
        self.assertEqual(
            parse_data_tile("./@@example.tile3"),
            "example.tile3",
        )
        self.assertEqual(
            parse_data_tile("/@@example.tile4"),
            "example.tile4",
        )
        self.assertEqual(
            parse_data_tile("./example.tile5"),
            "example.tile5",
        )

        # If we cannot parse it, try not to fail.
        self.assertIsNone(parse_data_tile("./@@bad.tile1//"))
        self.assertIsNone(parse_data_tile(""))
        self.assertIsNone(parse_data_tile("./"))
        self.assertIsNone(parse_data_tile("@@bad.tile2/"))

    def test_get_tile_types_from_text(self):
        from collective.tileindex.utils import get_tile_types_from_text

        persistent_tile = """
            <div data-tile="./@@plone.app.standardtiles.html/some-tile-uid"></div>"""
        transient_tile = """
            <div data-tile="./@@plone.app.standardtiles.field?field=IDublinCore-title">
            </div>"""
        self.assertEqual(
            get_tile_types_from_text(persistent_tile),
            {"plone.app.standardtiles.html"},
        )
        self.assertEqual(
            get_tile_types_from_text(transient_tile),
            {"plone.app.standardtiles.field"},
        )

        # get_tile_types_from_text returns a set.  Let's sort this to make it easier to compare.

        def get_tiles(content):
            return sorted(get_tile_types_from_text(content))

        self.assertEqual(
            get_tiles(persistent_tile),
            ["plone.app.standardtiles.html"],
        )
        self.assertEqual(
            get_tiles(transient_tile),
            ["plone.app.standardtiles.field"],
        )
        self.assertEqual(
            get_tiles(persistent_tile + transient_tile),
            [
                "plone.app.standardtiles.field",
                "plone.app.standardtiles.html",
            ],
        )
        self.assertEqual(
            get_tiles(persistent_tile * 10),
            [
                "plone.app.standardtiles.html",
            ],
        )
        # I have a doubt about which beautifulsoup parser we should use.
        # Some parsers fail on non-closing tags.  So try it.
        self.assertEqual(
            get_tiles(persistent_tile + '<br><img src="me.jpeg.">' + transient_tile),
            [
                "plone.app.standardtiles.field",
                "plone.app.standardtiles.html",
            ],
        )



from .upgrades import add_indexes
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles:
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            "collective.tileindex:uninstall",
        ]

    def getNonInstallableProducts(self):
        """Hide package from site-creation and quickinstaller."""
        return []


def post_install(context):
    """Post install script"""
    # Do something at the end of the installation of this package.
    add_indexes(context)


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



from Products.CMFCore.utils import getToolByName

import logging


logger = logging.getLogger(__name__)


def add_indexes(context):
    """Add indexes."""
    try:
        import plone.app.blocks  # noqa
    except ImportError:
        logger.info(
            "plone.app.blocks package not found. This means you have no tiles "
            "and do not need the tile_types index. You may have Volto blocks "
            "and can use our control panel to search the block_types index."
        )
        return

    catalog = getToolByName(context, "portal_catalog")
    indexes = catalog.indexes()

    wanted_indexes = [
        ("tile_types", "KeywordIndex"),
    ]

    indexables = []
    for index_name, index_type in wanted_indexes:
        if index_name not in indexes:
            catalog.addIndex(index_name, index_type)
            indexables.append(index_name)
            logger.info("Added {} for {}.".format(index_type, index_name))
    if len(indexables) > 0:
        logger.info("Indexing new indexes %s.", ", ".join(indexables))
        # I was pretty sure this also updated the catalog brains, but
        # it does not...
        catalog.manage_reindexIndex(ids=indexables)



from collective.tileindex.utils import get_tile_types_from_obj
from functools import cached_property
from Products.Five.browser import BrowserView


class TileTypesView(BrowserView):
    @cached_property
    def tile_types(self):
        tiles = get_tile_types_from_obj(self.context)
        if not tiles:
            return
        return sorted(tiles)



from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer

import collective.tileindex


class CollectiveTileindexLayer(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=collective.tileindex)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "collective.tileindex:default")


COLLECTIVE_TILEINDEX_FIXTURE = CollectiveTileindexLayer()


COLLECTIVE_TILEINDEX_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_TILEINDEX_FIXTURE,),
    name="CollectiveTileindexLayer:IntegrationTesting",
)


COLLECTIVE_TILEINDEX_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_TILEINDEX_FIXTURE,),
    name="CollectiveTileindexLayer:FunctionalTesting",
)



from functools import cached_property
from plone import api
from Products.Five.browser import BrowserView


class TileTypesOverview(BrowserView):
    @cached_property
    def tile_index(self):
        catalog = api.portal.get_tool(name="portal_catalog")
        return catalog.Indexes.get("tile_types")

    @cached_property
    def block_index(self):
        catalog = api.portal.get_tool(name="portal_catalog")
        return catalog.Indexes.get("block_types")

    @cached_property
    def tileorblock(self):
        if self.tile_index and not self.block_index:
            return "tile"
        elif not self.tile_index and self.block_index:
            return "block"
        return

    @cached_property
    def indexname(self):
        return f"{self.tileorblock}_types"

    @cached_property
    def indexmethod(self):
        return f"{self.tileorblock}_index"

    @cached_property
    def numObjects(self):
        return getattr(self, self.indexmethod).numObjects()

    @cached_property
    def alphabetical(self):
        return sorted(getattr(self, self.indexmethod).uniqueValues())

    @cached_property
    def numerical(self):
        items = sorted(
            [
                (len(value), key)
                for (key, value) in getattr(self, self.indexmethod).items()
            ],
            reverse=True,
        )
        result = []
        for item in items:
            result.append({"number": item[0], "tile": item[1]})
        return result

    @cached_property
    def search_results(self):
        tile = self.request.get("tile")
        if not tile:
            return (None, None)
        unpublished = []
        published = []
        query = {"sort_on": "path", "sort_order": "ascending"}
        query[self.indexname] = tile
        for item in api.content.find(**query):
            if item["review_state"] == "published":
                published.append(item)
            else:
                unpublished.append(item)
        return (published, unpublished)



from collective.tileindex.testing import COLLECTIVE_TILEINDEX_FUNCTIONAL_TESTING
from collective.tileindex.testing import COLLECTIVE_TILEINDEX_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class IndexerIntegrationTest(unittest.TestCase):
    layer = COLLECTIVE_TILEINDEX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_dummy(self):
        self.assertTrue(True)


class IndexerFunctionalTest(unittest.TestCase):
    layer = COLLECTIVE_TILEINDEX_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_dummy(self):
        self.assertTrue(True)



from bs4 import BeautifulSoup


try:
    from plone.app.blocks.layoutbehavior import ILayoutAware
except ImportError:
    # plone.app.blocks is a conditional dependency and is only needed it you
    # want to search for tiles.  If you use collective.tileindex for its
    # Volto blocks support, you do not need this part.
    # Note that Volto blocks and plone.app.blocks are totally unrelated.
    ILayoutAware = None

try:
    from plone.base.utils import base_hasattr
except ImportError:
    # BBB Plone 5.2
    from Products.CMFPlone.utils import base_hasattr


def parse_data_tile(dt):
    """Parse a data-tile attribute.

    Persistent tiles look like this:

        ./@@plone.app.standardtiles.html/some-tile-uid

    Transient tiles look like this:

        ./@@plone.app.standardtiles.field?field=IDublinCore-title

    We return a string like this, so without the "@@":

        plone.app.standardtiles.html
    """
    items = dt.split("/")
    if len(items) == 3:
        tile = items[1]
    elif len(items) == 2:
        tile = items[1].split("?")[0]
    else:
        return
    return tile.strip("@") or None


def get_tile_types_from_text(text):
    soup = BeautifulSoup(text, features="lxml")
    # We search for content like this.
    # Persistent tile:
    # <div data-tile="./@@plone.app.standardtiles.html/some-tile-uid"></div>
    # Transient tile:
    # <div data-tile="./@@plone.app.standardtiles.field?field=IDublinCore-title"></div>
    divs = soup.find_all(attrs={"data-tile": True})
    tile_types = set()
    for dt in [d["data-tile"] for d in divs]:
        tile = parse_data_tile(dt)
        if tile:
            tile_types.add(tile)
    return tile_types


def get_tile_types_from_obj(obj):
    if ILayoutAware is None:
        return
    if not base_hasattr(obj, "__annotations__"):
        return
    if obj.getProperty("layout", "") != "layout_view":
        return

    # Get the tiles from the content layout.
    layout = ILayoutAware(obj)
    content_layout = layout.content_layout()
    if not content_layout:
        return
    return get_tile_types_from_text(content_layout)



from collective.tileindex.testing import COLLECTIVE_TILEINDEX_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getMultiAdapter

import unittest


class ViewsIntegrationTest(unittest.TestCase):
    layer = COLLECTIVE_TILEINDEX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_tile_types_is_registered_document(self):
        api.content.create(self.portal, "Document", "front-page")
        view = getMultiAdapter(
            (self.portal["front-page"], self.portal.REQUEST), name="tile-types-view"
        )
        self.assertIsNone(view.tile_types)

    def test_tile_types_is_registered_folder(self):
        api.content.create(self.portal, "Folder", "other-folder")
        view = getMultiAdapter(
            (self.portal["other-folder"], self.portal.REQUEST), name="tile-types-view"
        )
        self.assertIsNone(view.tile_types)



"""Installer for the collective.tileindex package."""

from setuptools import find_packages
from setuptools import setup


long_description = "\n\n".join(
    [
        open("README.rst").read(),
        open("CONTRIBUTORS.rst").read(),
        open("CHANGES.rst").read(),
    ]
)


setup(
    name="collective.tileindex",
    version="1.0.0a4",
    description="An add-on for Plone",
    long_description=long_description,
    # Get more from https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: 6.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="Python Plone CMS",
    author="Zest Software",
    author_email="info@zestsoftware.nl",
    url="https://github.com/collective/collective.tileindex",
    project_urls={
        "PyPI": "https://pypi.org/project/collective.tileindex/",
        "Source": "https://github.com/collective/collective.tileindex",
        "Tracker": "https://github.com/collective/collective.tileindex/issues",
        # 'Documentation': 'https://collective.tileindex.readthedocs.io/en/latest/',
    },
    license="GPL version 2",
    packages=find_packages("src", exclude=["ez_setup"]),
    namespace_packages=["collective"],
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.7",
    install_requires=[
        "setuptools",
        "plone.api>=1.8.4",
    ],
    extras_require={
        "test": [
            "plone.app.testing",
            "plone.testing>=5.0.0",
            "plone.app.blocks",
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



from collective.tileindex.utils import get_tile_types_from_obj
from plone.app.blocks.layoutbehavior import ILayoutBehaviorAdaptable
from plone.indexer.decorator import indexer


@indexer(ILayoutBehaviorAdaptable)
def tile_types(obj):
    """Calculate and return the value for the indexer"""
    return get_tile_types_from_obj(obj)
